/*
 * Renderer 10. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.util.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   This file just shows what Model_1 looks like.
<p>
   The tree for this scene is shown below.
<p>
   Remember that every position node in the tree contains a matrix,
   a model and a list of nested positions. The model may be empty,
   and the list of nested positions may also be empty, but the matrix
   cannot be "empty" (if you don't give it a value, then it is the
   identity matrix, I).
<p>
<pre>{@code
           Scene
          /     \
         /       \
    Camera     List<Position>
                     |
                     |
                  Position
                 /   |    \
                /    |     \
          Matrix   Model_1   List<Position>
            R                      |
                                 empty
}</pre>
*/
public class ShowModel_1
{
   public static void main(String[] args)
   {
      final Scene scene = new Scene("ShowModel_1",
                                    Camera.projOrtho(-3, +3, -3, +3));

      // Create an instance of Model_1.
      final Model m = new Model_1();

      // Set the model's color.
      ModelShading.setColor(m, Color.black);

      // Create the top level Position holding the model.
      final Position p0 = new Position(m, "top");

      // Add the top level Position to the Scene.
      scene.addPosition(p0);

      DrawSceneGraph.draw(scene, "SG_ShowModel_1");
      DrawSceneGraphTopologicalSort.draw(scene, "SG_ShowModel_1_TS");


      // Create a FrameBuffer to render our Scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.white);

      //PipelineLogger.debug = true;

      for (int i = 0; i <= 36; ++i)
      {
         scene.setPosition(0,
                  scene.getPosition(0).transform( Matrix.rotateZ(10*i) ));

         // Render again.
         fb.clearFB();
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_ShowModel_1_Frame%02d.ppm", i));
      }
   }
}
